# 解法

本ソルバーはポートフォリオ型であり、以下の解法を使い分けている。

1. Depth-First Search  
2. Meet in the middle  
3. Simpath[1]  
    3-a. default edge ordering  
    3-b. community based edge ordering

使い分けの基準は次の通りである。

1. 最大パス長 $l$ が $14$ 以下: アルゴリズム 2
2. 最大パス長 $l$ が $15$ 以上: アルゴリズム 1, 3-a, 3-b の並列実行

## Depth-First Search

答えが小さなケースに対しては愚直な深さ優先探索による解法が有効である。

all-pairs インスタンスに対しては、始点 $s$ を全探索し、終点 $t$ が $s\lt t$ を満たすような単純パスを数えればよい。各 $s$ に対して DFS を行う部分は容易に並列化可能である。

愚直な深さ優先探索を、neighborhood diversity [2] が小さなグラフに対してより高速に動作するよう改善することができる。neighborhood diversity は、頂点集合 $V$ 上の同値関係 $u \sim _ {\mathrm{nd}} v \overset{\text{def}}{\iff} N(u)\setminus\lbrace v\rbrace = N(v)\setminus\lbrace u\rbrace$ による同値類の個数として定義される。ここで $N(u)$ は頂点 $u$ に隣接する頂点の集合を表す。

単純パス $P$ を頂点列 $(P _ 0, P _ 1, \ldots, P _ k)$ で表す。 $P _ i \sim _ {\mathrm{nd}} P _ j$ なる $0\leq i\lt j\leq k$ が含まれるとする。このとき $P$ において $u$ と $v$ を入れ替えてできる頂点列 $(P _ 0, \ldots, P _ {i - 1}, P _ j, P _ {i + 1}, \ldots, P _ {j - 1}, P _ i, P _ {j + 1}, \ldots, P _ k)$ は単純パスである。これは $P$ が単純であることと $\sim _ {\mathrm{nd}}$ の定義から従う。

そこで、入力グラフ $G(V, E)$ に対して新たなグラフ $G'(V/{\sim _ {\mathrm{nd}}}, E')$ を構築する。ここで $E'$ は次のように定義される。 $G'$ は多重辺を含まないが自己ループを含みうることに注意せよ。

$$E' \coloneqq \lbrace \lbrace \lbrack u\rbrack, \lbrack v\rbrack \rbrace \mid \exists a\in \lbrack u\rbrack,\exists b\in \lbrack v\rbrack \text{ s.t. } \lbrace a,b\rbrace \in E\rbrace.$$

$G$ 上の単純パス $P=(P _ 0, \ldots, P _ k)$ に対して $\lbrack P\rbrack \coloneqq (\lbrack P _ 0\rbrack, \ldots, \lbrack P _ k\rbrack)$ と定義する。 $\lbrack P\rbrack$ は $G'$ 上の単純とは限らないパスとなる。

さて、 $G'$ 上の単純とは限らないパス $Q$ に対して $\lbrack P\rbrack = Q$ を満たす $G$ 上の単純パス $P$ の個数を数えることを考える。 $W _ {\lbrack u\rbrack} \coloneqq \lvert \lbrack u\rbrack\rvert$ および $C _ {\lbrack u\rbrack} \coloneqq \lvert \lbrace i \mid Q _ i = \lbrack u \rbrack \rbrace \rvert$ と定義するとその個数は次の式で表される。

$$\prod _ {\lbrack u \rbrack \in V/{\sim _ {\mathrm{nd}}}} {} _ {W _ {\lbrack u\rbrack}} P _ {C _ {\lbrack u\rbrack}} = \prod _ {\lbrack u \rbrack \in V/{\sim _ {\mathrm{nd}}}} \left(W _ {\lbrack u\rbrack} (W _ {\lbrack u\rbrack} - 1) \cdots (W _ {\lbrack u\rbrack} - C _ {\lbrack u \rbrack} + 1)\right).$$

また $\lbrack P\rbrack = Q$ を満たす $G$ 上の $s{-}t$ 単純パス $P$ の個数は次で表される。

$$\begin{cases} 0 & \text{if } s\notin Q _ 0 \vee t \notin Q _ k \newline
\displaystyle {} _ {W _ {\lbrack s\rbrack} - 2} P _ {C _ {\lbrack s\rbrack} - 2} \cdot \prod _ {\lbrack u \rbrack \in (V/{\sim _ {\mathrm{nd}}})\setminus\lbrace \lbrack s\rbrack \rbrace} {} _ {W _ {\lbrack u\rbrack}} P _ {C _ {\lbrack u\rbrack}} & \text{if } s\in Q _ 0 \wedge t \in Q _ k \wedge Q _ 0 = Q _ k \newline
\displaystyle {} _ {W _ {\lbrack s\rbrack} - 1} P _ {C _ {\lbrack s\rbrack} - 1} \cdot {} _ {W _ {\lbrack t\rbrack} - 1} P _ {C _ {\lbrack t\rbrack} - 1} \cdot \prod _ {\lbrack u \rbrack \in (V/{\sim _ {\mathrm{nd}}})\setminus\lbrace \lbrack s\rbrack, \lbrack t\rbrack \rbrace} {} _ {W _ {\lbrack u\rbrack}} P _ {C _ {\lbrack u\rbrack}} & \text{if } s\in Q _ 0 \wedge t \in Q _ k \wedge Q _ 0 \neq Q _ k \newline
\end{cases}$$

以上より $G'$ 上の単純とは限らないパスを重み付きで数え上げることで元の問題を解くことができる。なお、最大パス長の制約に関しては $P$ と $\lbrack P \rbrack$ は長さが等しいため容易に処理可能である。

## Meet in the middle

単純パス $P$ を頂点列 $(P _ 0, P _ 1, \ldots, P _ k)$ で表す。 $P$ を長さ $\lceil k/2\rceil$ の単純パス $(P _ 0, P _ 1, \ldots, P _ {\lceil k/2\rceil})$ と 長さ $\lfloor k/2\rfloor$ の単純パス $(P _ {\lceil k/2\rceil}, P _ {\lceil k/2\rceil + 1}, \ldots, P _ k)$ を繋いで得ることを考える。

### all-pairs

$1\leq k \leq l$ なる整数 $k$ と中点となる頂点 $x$ を固定する。 $x$ を始点とする長さ $i$ の単純パスの集合を $\mathcal{P} _ {x,i}$ とする。

$L = (x = L _ 0, L _ 1, \ldots, L _ {\lceil k / 2 \rceil}) \in \mathcal{P} _ {x,\lceil k / 2 \rceil}$ と $R = (x = R _ 0, R _ 1, \ldots, R _ {\lfloor k / 2 \rfloor})\in \mathcal{P} _ {x,\lfloor k / 2 \rfloor}$ を繋げて得られる単純とは限らないパス $P = (L _ {\lceil k / 2 \rceil}, L _ {\lceil k / 2 \rceil - 1}, \ldots, L _ 1, x, R _ 1, R _ 2, \ldots, R _ {\lfloor k / 2\rfloor})$ を考える。 $V(L)\cap V(R) = \lbrace x\rbrace$ のとき、またそのときに限り $P$ は単純である。ここで $V(S)$ は頂点列 $S$ に現れる頂点の集合を表す。

$L\in \mathcal{P} _ {x,\lceil k / 2 \rceil}$ に対して、 $P$ が単純となるような $R\in\mathcal{P} _ {x,\lfloor k / 2 \rfloor}$ の個数を $C _ {x,k}(L)$ とする。包除原理より $C _ {x,k}(L)$ は次のように計算できる。

$$C _ {x,k}(L) = \sum _ {S\subseteq (V(L)\setminus\lbrace x\rbrace)} (-1) ^ {\lvert S\rvert} \cdot \lvert \lbrace R\in \mathcal{P} _ {x,\lfloor k / 2 \rfloor} \mid (V(R)\setminus\lbrace x\rbrace) \supseteq S \rbrace \rvert.$$

和の計算時に毎回 $D _ {x, k}(S) = \lvert \lbrace R\in \mathcal{P} _ {x,\lfloor k / 2 \rfloor} \mid (V(R)\setminus\lbrace x\rbrace) \supseteq S \rbrace\rvert$ を愚直に計算するのは効率が悪い。そこで、全ての $S\subseteq V$ に対する $D _ {x, k}(S)$ を以下の手順で前計算する。

1. 空の hash map として $D _ {x, k}$ を初期化する。
2. 各 $R\in \mathcal{P} _ {x,\lfloor k / 2 \rfloor}$ に対して、次を行う。
      1. 各 $S\subseteq V(R) \setminus\lbrace x\rbrace$ に対して、次を行う。
            - $S$ が $D _ {x, k}$ のキーとして含まれていなければ $D _ {x, k}(S) \gets 1$ とする。
            - 含まれていれば、$D _ {x, k}(S) \gets D _ {x, k}(S) + 1$ と更新する。

この前計算は (expected) $O(\lvert \mathcal{P} _ {x,\lfloor k / 2 \rfloor}\rvert k 2 ^ {\lfloor k / 2\rfloor})$ 時間で可能であり、全ての $L \in \mathcal{P} _ {x,\lceil k / 2 \rceil}$ に対する $C _ {x,k}(L)$ の計算は (expected) $O(\lvert \mathcal{P} _ {x,\lceil k / 2 \rceil}\rvert k 2 ^ {\lceil k / 2\rceil})$ 時間で可能である。

今回の問題では向きだけが異なる $2$ つのパスは同じパスとして数える必要がある。従って、答えは $\displaystyle \dfrac{1}{2}\sum _ {x\in V} \sum _ {k = 1} ^ {l} \sum _ {L \in\mathcal{P} _ {x, \lceil k / 2\rceil}} C _ {x,k}(L)$ と表される。

さらに、前節の考察から、答えは $\displaystyle \dfrac{1}{2}\sum _ {\lbrack x\rbrack \in V/{\sim _ {\mathrm{nd}}}} W _ {\lbrack x\rbrack} \sum _ {k = 1} ^ {l} \sum _ {L \in\mathcal{P} _ {x, \lceil k / 2\rceil}} C _ {x,k}(L)$ とも表せる。

なお、実装上は $\mathcal{P} _ {x, \ast}$ を計算しながら同時に包除原理による数え上げも行うことで高速化している。

また $x$ を全探索する部分は容易に並列化可能である。

### one-pair

$1\leq k \leq l$ なる整数 $k$ と中点となる頂点 $x$ を固定する。長さ $i$ の単純 $x{-}s$ パスの集合を $\mathcal{L} _ {x,i}$ とする。また、長さ $i$ の単純 $x{-}t$ パスの集合を $\mathcal{R} _ {x,i}$ とする。同様に包除原理を適用することで、答えは以下で表される。

$$\sum _ {x\in V} \sum _ {k = 1} ^ l \sum _ {L \in \mathcal{L} _ {x,\lceil k / 2 \rceil}} \sum _ {S \subseteq V(L)\setminus\lbrace x\rbrace} (-1) ^ {\lvert S\rvert} \cdot \lvert \lbrace R\in \mathcal{R} _ {x,\lfloor k / 2 \rfloor} \mid (V(R)\setminus\lbrace x\rbrace) \supseteq S \rbrace \rvert.$$

など、all-pairs の場合と同様にして neighborhood diversity が小さなグラフに対して効率化を図ることができる。

## Simpath [1]

Simpath はゼロサプレス型二分決定木 (Zero-supressed Binary Decision Diagram; ZDD) [3] を用いて単純パスを列挙するアルゴリズムである。詳細は割愛する。本ソルバーでは $2$ 種類の辺順序を試している。

### default order

入力データが既に整理されている場合、辺順序は変更せずそのまま用いた方が効率が良い場合がある。

### community-based order

以下の手順で辺を並び替えた。

1. Girvan-Newman algorithm [4] によって頂点集合を幾つかの community に分割する。Girvan-Newman algorithm ではいくつかの分割の候補が得られるが、そのうち modularity [5] が最大のものを採用する。
2. 同じ community に属する頂点の id が連続するように頂点番号を振り直す。community 間の順序は、community を縮約したグラフの DFS 木の preorder とする。DFS 木の根は、one-pair インスタンスに対しては端点 $s$ が属する community とした。all-pairs インスタンスに対しては無作為に選んだ。
3. 辺 $\lbrace u, v\rbrace$ を $(\min(u,v),\max(u,v))$ の昇順に並び替える。

並べ替えの例は [description.pdf](./description.pdf) の Figure 1,2,3 を参照。

## Reference

[1] Donald E. Knuth.The Art of Computer Programming, Volume 4, Fascicle1: Bitwise Tricks & Techniques; Binary Decision Diagrams. Addison-Wesley Professional, 12th edition, 2009.

[2] Michael Lampis. Algorithmic meta-theorems for restrictions of treewidth.InProceedings of the 18th Annual European Conference on Algorithms:Part I, ESA’10, page 549–560, Berlin, Heidelberg, 2010. Springer-Verlag.

[3] Shin-ichi Minato. Zero-suppressed bdds for set manipulation in combina-torial problems. InProceedings of the 30th International Design Automa-tion Conference, DAC ’93, page 272–277, New York, NY, USA, 1993.Association for Computing Machinery.

[4] M. Girvan and M. E. J. Newman. Community structure in social andbiological networks.Proceedings of the National Academy of Sciences,99(12):7821–7826, 2002.

[5] M. E. J. Newman and M. Girvan. Finding and evaluating communitystructure in networks.Phys. Rev. E, 69:026113, Feb 2004.6
