#pragma once

#include <cassert>
#include <deque>
#include <limits>
#include <vector>

/**
 * @brief run BFS and calculate distance from the vertex s to all vertices
 * @param g adjacency list
 * @param s source vertex
 * @return std::vector<int> d with length g.size(), which satisfies d[i] = dist(s, i)
 */
std::vector<int> bfs(const std::vector<std::vector<int>> &g, int s) {
    const int n = g.size();
    assert(0 <= s and s < n);

    constexpr int inf = std::numeric_limits<int>::max() / 2;

    std::vector<int> dist(n, inf);
    dist[s] = 0;

    std::deque<int> dq{s};
    while (dq.size()) {
        int u = dq.front();
        dq.pop_front();
        for (int v : g[u]) if (dist[v] == inf) {
            dist[v] = dist[u] + 1;
            dq.push_back(v);
        }
    }
    return dist;
}