#pragma once

#include <iostream>

// colored logging to stderr
namespace colorlog {
    constexpr int BLACK = 30;
    constexpr int RED = 31;
    constexpr int GREEN = 32;
    constexpr int YELLOW = 33;
    constexpr int BLUE = 34;
    constexpr int MAGENTA = 35;
    constexpr int CYAN = 36;
    constexpr int WHITE = 37;

    template <int color_code>
    struct Logger {
        template <typename T>
        void log(T&& v, bool do_flush = true) const {
            *this << v << '\n';
            if (do_flush) flush();
        }
        template <typename T>
        void operator()(T&& v, bool do_flush = true) const { log(std::forward<T>(v), do_flush); }
        template <typename T>
        const Logger& operator<<(T&& v) const {
            std::cerr << "\033[" << color_code << "m" << v << "\033[m";
            return *this;
        }
        void flush() {
            std::cerr.flush();
        }
    };

    Logger<WHITE> info;
    Logger<GREEN> success;
    Logger<YELLOW> warning;
    Logger<RED> error;
}