#pragma once

#include <cstdint>
#include <iostream>

/**
 * @brief Z/mZ
 * @tparam MOD modulo
 */
template <__uint128_t MOD>
struct modint {
    static constexpr __uint128_t mod = MOD;

    __uint128_t val;

    modint() : val(0) {}
    modint(__uint128_t val) : val(val % mod) {}

    static modint raw(__uint128_t val) {
        modint x;
        x.val = val;
        return x;
    }

    friend modint& operator+=(modint &x, const modint &y) {
        x.val += y.val;
        x.val -= mod * ((mod - x.val) >> 127);
        return x;
    }
    friend modint operator+(const modint &x, const modint &y) {
        modint z = x;
        z += y;
        return z;
    }

    friend std::ostream& operator<<(std::ostream& out, const modint &x) {
        return out << x.val;
    }
};