#pragma once

#include <iostream>
#include <sstream>

#include "colorlog.hpp"

/**
 * @brief to output progress using logger
 * @tparam color_code colorlog::GREEN, colorlog::YELLOW, ...
 */
template <int color_code>
struct ProgressBar {
    ProgressBar(const colorlog::Logger<color_code> &logger, const std::string &tag, int width = 70) : _logger(logger), _tag(tag), _width(width) {}

    void set_bar_width(int width) {
        _width = width;
    }
    void set_tag(const std::string &tag) {
        _tag = tag;
    }
    void write(int done, int all) {
        assert(0 <= done and done <= all);

        const int pos = _width * done / all;

        std::ostringstream bar;

        bar << '[';
        bar << std::string(pos, '=');
        if (pos != _width) bar << '>';
        bar << std::string(std::max(0, _width - (pos + 1)), ' ');
        bar << ']';

        bar << ' ' << done << '/' << all << " (" << int(100 * done / all) << "%)";

        _logger << _tag << ' ' << bar.str() << (done == all ? '\n' : '\r');
        _logger.flush();
    }
private:
    colorlog::Logger<color_code> _logger;
    std::string _tag;
    int _width;
};