from dataclasses import dataclass
from typing import List, Optional, TextIO, Tuple

@dataclass
class Instance:
    V: int
    E: List[Tuple[int, int]]
    L: int
    s: Optional[int]
    t: Optional[int]

    def __post_init__(self):
        assert 1 <= self.V

        for u, v in self.E:
            assert 1 <= u <= self.V
            assert 1 <= v <= self.V

        assert 1 <= self.L

        assert (self.s is None) == (self.t is None)

    @staticmethod
    def read_DIMACS(f: TextIO):
        N = M = -1
        E = list()
        L = -1
        s = None
        t = None

        for line in f:
            if line.startswith('p'):
                N, M = map(int, line.split()[2:])
            elif line.startswith('e'):
                u, v = map(int, line.split()[1:])
                E.append((u, v))
            elif line.startswith('l'):
                L = int(line.split()[1])
            elif line.startswith('t'):
                s, t = map(int, line.split()[1:])

        assert len(E) == M

        return Instance(V=N, E=E, L=L, s=s, t=t)
    
    def DIMACS(self):
        lines = []
        lines.append(f'p edge {self.V} {len(self.E)}')
        for u, v in self.E:
            lines.append(f'e {u} {v}')
        lines.append(f'l {self.L}')
        if self.s is not None:
            lines.append(f't {self.s} {self.t}')
        return '\n'.join(lines)
