#include <algorithm>
#include <numeric>
#include <iostream>
#include <iomanip>

#include "include/instance.hpp"
#include "include/solve_frontier.hpp"

#include <chrono>

#include "include/util/io.hpp"
#include "include/solve_naive_dfs.hpp"

int main() {
    const __uint128_t answer = [] {
        instance problem;
        problem.read_DIMACS(std::cin);
        problem.l = std::min(problem.n - 1, problem.l);

        auto start_time = std::chrono::system_clock::now();

        std::vector<__uint128_t> ans = [&] {
            if (problem.s != -1) {
                problem = problem.reduce();
            }
            return solve_dfs<__uint128_t>(problem);
        }();

        __uint128_t sum = std::accumulate(ans.begin(), ans.end(), __uint128_t(0));
        std::cerr << "Answer = " << sum << std::endl;
        std::cerr << "Details:" << std::endl;
        for (int i = 0; i < int(ans.size()); ++i) {
            std::cerr << "    Path size " << std::setw(2) << i << ": " << ans[i] << std::endl;
        }

        auto end_time = std::chrono::system_clock::now();

        auto elapsed_sec = std::chrono::duration_cast<std::chrono::milliseconds>(end_time - start_time).count();

        std::cerr << "Solved Time: " << elapsed_sec << " ms" << std::endl;

        return sum;
    }();

    std::cout << answer << std::endl;
}