import argparse
import logging
from pathlib import Path
import subprocess

TEST_BENCH = Path('./test_bench')
SOLVER_EXPECTED = Path('./solve_graphillion.out')

def test(input_file: Path, solver_cmd: str):
    with input_file.open(mode='r') as f:
        expected = subprocess.run(
            [f'{SOLVER_EXPECTED.resolve()}'],
            stdin=f,
            stdout=subprocess.PIPE,
            check=True
        ).stdout.strip()
    logging.info(f'expected: {expected}')

    with input_file.open(mode='r') as f:
        actual = subprocess.run(
            solver_cmd.split(),
            stdin=f,
            stdout=subprocess.PIPE,
            check=True
        ).stdout.strip()
    logging.info(f'actual  : {actual}')

    assert int(actual) == int(expected)

if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument(type=str, dest='cmd')
    args = parser.parse_args()

    solver_cmd = args.cmd

    logging.basicConfig(filename='./test/verification.log', level=logging.INFO)
    logging.info(f'Running all_pairs Tests')
    for input_file in sorted(TEST_BENCH.glob('all_pairs/*.col'), key=str):
        logging.info(f'Running Test {input_file}')
        test(input_file, solver_cmd)

    logging.info(f'Running one_pair Tests')
    for input_file in sorted(TEST_BENCH.glob('one_pair/*.col'), key=str):
        logging.info(f'Running Test {input_file}')
        test(input_file, solver_cmd)
